@@ -70,12 +70,12 @@ gboolean genius_is_gui = FALSE;
 GelOutput *main_out = NULL;
 
 void (*evalnode_hook)(void) = NULL;
-int run_hook_every = 1000;
 void (*statechange_hook)(calcstate_t) = NULL;
 
 typedef struct {
 	char *category;
 	char *name;
+	gboolean internal;
 	GSList *funcs;
 } HelpCategory;
 static GSList *categories = NULL;
@@ -89,7 +89,7 @@ static gboolean got_end_too_soon = FALSE;
 GHashTable *uncompiled = NULL;
 
 /* stack ... has to be global:-( */
-GSList *evalstack=NULL;
+GSList *gel_parsestack=NULL;
 
 /*error .. global as well*/
 GeniusError error_num = NO_ERROR;
@@ -190,7 +190,7 @@ get_category_name (const char *category)
 	if (cat == NULL || cat->name == NULL)
 		return category;
 	else
-		return cat->name;
+		return _(cat->name);
 }
 
 /* gets undocumented functions */
@@ -282,11 +282,12 @@ get_undocumented (void)
 }
 
 void
-new_category (const char *category, const char *name)
+new_category (const char *category, const char *name, gboolean internal)
 {
 	HelpCategory *cat = get_category (category, TRUE /* insert */);
 	g_free (cat->name);
 	cat->name = g_strdup (name);
+	cat->internal = internal;
 }
 
 static void
@@ -404,19 +405,6 @@ add_description (const char *func, const char *desc)
 	g_free (d);
 }
 
-const char *
-get_description(const char *func)
-{
-	GelHelp *help;
-
-	help = get_help (func, FALSE /* insert */);
-	if (help == NULL ||
-	    help->description == NULL)
-		return  "";
-	else
-		return help->description;
-}
-
 void
 whack_help (const char *func)
 {
@@ -576,7 +564,16 @@ appendoper(GelOutput *gelo, GelETree *n)
 
 	switch(n->op.oper) {
 		case E_SEPAR:
-			append_binaryoper(gelo,";",n); break;
+			gel_output_string (gelo,"(");
+			r = n->op.args;
+			while (r != NULL) {
+				gel_print_etree (gelo, r, FALSE);
+				r = r->any.next;
+				if (r != NULL)
+					gel_output_string (gelo, ";");
+			}
+			gel_output_string (gelo,")");
+			break;
 		case E_EQUALS:
 			append_binaryoper(gelo,"=",n); break;
 		case E_PARAMETER:
@@ -599,14 +596,18 @@ appendoper(GelOutput *gelo, GelETree *n)
 			break;
 		case E_ABS:
 			GET_L(n,l);
-			gel_output_string(gelo,"(|");
+			gel_output_string(gelo,"|");
 			gel_print_etree(gelo, l, FALSE);
-			gel_output_string(gelo,"|)");
+			gel_output_string(gelo,"|");
 			break;
 		case E_PLUS:
 			append_binaryoper(gelo,"+",n); break;
+		case E_ELTPLUS:
+			append_binaryoper(gelo,".+",n); break;
 		case E_MINUS:
 			append_binaryoper(gelo,"-",n); break;
+		case E_ELTMINUS:
+			append_binaryoper(gelo,".-",n); break;
 		case E_MUL:
 			append_binaryoper(gelo,"*",n); break;
 		case E_ELTMUL:
@@ -744,17 +745,17 @@ appendoper(GelOutput *gelo, GelETree *n)
 			GET_LR(n,l,r);
 			gel_output_string(gelo,"(");
 			gel_print_etree (gelo, l, FALSE);
-			gel_output_string(gelo,"@[");
+			gel_output_string(gelo,"@(");
 			gel_print_etree (gelo, r, FALSE);
-			gel_output_string(gelo,",])");
+			gel_output_string(gelo,",))");
 			break;
 		case E_GET_COL_REGION:
 			GET_LR(n,l,r);
 			gel_output_string(gelo,"(");
 			gel_print_etree (gelo, l, FALSE);
-			gel_output_string(gelo,"@[,");
+			gel_output_string(gelo,"@(,");
 			gel_print_etree (gelo, r, FALSE);
-			gel_output_string(gelo,"])");
+			gel_output_string(gelo,"))");
 			break;
 
 		case E_QUOTE:
@@ -926,7 +927,6 @@ appendoper(GelOutput *gelo, GelETree *n)
 		case E_DIRECTCALL:
 		case E_CALL:
 			GET_L(n,l);
-			gel_output_string(gelo,"(");
 			if (l->type==IDENTIFIER_NODE) {
 				gel_output_string (gelo, l->id.id->token);
 			} else if (l->type == FUNCTION_NODE &&
@@ -956,7 +956,7 @@ appendoper(GelOutput *gelo, GelETree *n)
 				gel_output_string(gelo,",");
 				gel_print_etree (gelo, li, FALSE);
 			}
-			gel_output_string(gelo,"))");
+			gel_output_string(gelo,")");
 			break;
 		case E_RETURN:
 			append_unaryoper(gelo,"return ",n); break;
@@ -1175,7 +1175,7 @@ appendpolynomial (GelOutput *gelo, GelETree *n)
 		int i = index[0] + (index[1]*stride) + (index[2]*stride*stride);
 		int j;
 		if (n->poly.indexes[i] != NULL &&
-		    mpw_sgn (n->poly.indexes[i]) != 0) {
+		    ! mpw_eql_ui (n->poly.indexes[i], 0)) {
 			char *p;
 			gboolean firstvar;
 
@@ -1335,21 +1335,27 @@ gel_print_etree (GelOutput *gelo,
 				gel_output_string (gelo, "...");
 
 			if G_LIKELY (f->type==GEL_USER_FUNC) {
-				gel_output_string(gelo,")=(");
+				gel_output_string(gelo,")=");
 				D_ENSURE_USER_BODY (f);
 				gel_print_etree (gelo, f->data.user, FALSE);
-				gel_output_string(gelo,"))");
+				gel_output_string(gelo,")");
 			} else {
 				/*variable and reference functions should
 				  never be in the etree*/
 				gel_errorout (_("Unexpected function type!"));
-				gel_output_string(gelo,")(?)");
+				gel_output_string(gelo,")(?))");
 			}
 			break;
 		}
 	case COMPARISON_NODE:
 		appendcomp(gelo,n);
 		break;
+	case BOOL_NODE:
+		if (n->bool_.bool_)
+			gel_output_string (gelo, "true");
+		else
+			gel_output_string (gelo, "false");
+		break;
 	default:
 		gel_errorout (_("Unexpected node!"));
 		gel_output_string(gelo,"(?)");
@@ -1662,10 +1668,11 @@ compile_funcs_in_dict (FILE *outfile, GSList *dict, gboolean is_extra_dict)
 		}
 		if (func->type == GEL_USER_FUNC) {
 			fprintf (outfile,
-				 "%c;%d;%s;%d;%d",
+				 "%c;%d;%s;%s;%d;%d",
 				 fs,
 				 (int)strlen (body),
 				 func->id->token,
+				 func->symbolic_id ? func->symbolic_id->token : "*",
 				 (int)func->nargs,
 				 (int)func->vararg);
 			for (l = func->named_args; l != NULL; l = l->next) {
@@ -1724,7 +1731,7 @@ compile_funcs_in_dict (FILE *outfile, GSList *dict, gboolean is_extra_dict)
 				g_free (s);
 			}
 		}
-		if (func->id->protected)
+		if (func->id->protected_)
 			fprintf (outfile,"P;%s\n",func->id->token);
 	}
 }
@@ -1778,13 +1785,14 @@ load_compiled_fp (const char *file, FILE *fp)
 	while ( ! break_on_next && fgets (buf, buf_size, fp) != NULL) {
 		char *p;
 		char *b2;
-		GelToken *tok;
+		GelToken *tok, *symbolic_tok = NULL;
 		int size, nargs, vararg;
 		gboolean extra_dict = FALSE;
 		gboolean parameter = FALSE;
 		int i;
 		GSList *li = NULL;
 		int type;
+		char *ptrptr;
 
 		gel_incr_file_info();
 
@@ -1804,7 +1812,7 @@ load_compiled_fp (const char *file, FILE *fp)
 			}
 		}
 
-		p = strtok(buf,";");
+		p = strtok_r (buf,";", &ptrptr);
 		if G_UNLIKELY (!p) {
 			gel_errorout (_("Badly formed record"));
 			continue;
@@ -1813,12 +1821,12 @@ load_compiled_fp (const char *file, FILE *fp)
 			continue;
 		} else if (*p == 'A') {
 			char *d;
-			p = strtok(NULL,";");
+			p = strtok_r (NULL,";", &ptrptr);
 			if G_UNLIKELY (!p) {
 				gel_errorout (_("Badly formed record"));
 				continue;
 			}
-			d = strtok(NULL,";");
+			d = strtok_r (NULL,";", &ptrptr);
 			if G_UNLIKELY (!d) {
 				gel_errorout (_("Badly formed record"));
 				continue;
@@ -1827,12 +1835,12 @@ load_compiled_fp (const char *file, FILE *fp)
 			continue;
 		} else if (*p == 'C') {
 			char *d;
-			p = strtok(NULL,";");
+			p = strtok_r (NULL,";", &ptrptr);
 			if G_UNLIKELY (!p) {
 				gel_errorout (_("Badly formed record"));
 				continue;
 			}
-			d = strtok(NULL,";");
+			d = strtok_r (NULL,";", &ptrptr);
 			if G_UNLIKELY (!d) {
 				gel_errorout (_("Badly formed record"));
 				continue;
@@ -1841,12 +1849,12 @@ load_compiled_fp (const char *file, FILE *fp)
 			continue;
 		} else if (*p == 'D') {
 			char *d;
-			p = strtok(NULL,";");
+			p = strtok_r (NULL,";", &ptrptr);
 			if G_UNLIKELY (!p) {
 				gel_errorout (_("Badly formed record"));
 				continue;
 			}
-			d = strtok(NULL,";");
+			d = strtok_r (NULL,";", &ptrptr);
 			if G_UNLIKELY (!d) {
 				gel_errorout (_("Badly formed record"));
 				continue;
@@ -1855,12 +1863,12 @@ load_compiled_fp (const char *file, FILE *fp)
 			continue;
 		} else if (*p == 'L') {
 			char *d, *h;
-			p = strtok(NULL,";");
+			p = strtok_r (NULL,";", &ptrptr);
 			if G_UNLIKELY (!p) {
 				gel_errorout (_("Badly formed record"));
 				continue;
 			}
-			d = strtok(NULL,";");
+			d = strtok_r (NULL,";", &ptrptr);
 			if G_UNLIKELY (!d) {
 				gel_errorout (_("Badly formed record"));
 				continue;
@@ -1871,12 +1879,12 @@ load_compiled_fp (const char *file, FILE *fp)
 			continue;
 		} else if (*p == 'H') {
 			char *d, *h;
-			p = strtok(NULL,";");
+			p = strtok_r (NULL,";", &ptrptr);
 			if G_UNLIKELY (!p) {
 				gel_errorout (_("Badly formed record"));
 				continue;
 			}
-			d = strtok(NULL,";");
+			d = strtok_r (NULL,";", &ptrptr);
 			if G_UNLIKELY (!d) {
 				gel_errorout (_("Badly formed record"));
 				continue;
@@ -1887,13 +1895,13 @@ load_compiled_fp (const char *file, FILE *fp)
 			continue;
 		} else if (*p == 'P') {
 			GelToken *tok;
-			p = strtok(NULL,";");
+			p = strtok_r (NULL,";", &ptrptr);
 			if G_UNLIKELY (!p) {
 				gel_errorout (_("Badly formed record"));
 				continue;
 			}
 			tok = d_intern(p);
-			tok->protected = 1;
+			tok->protected_ = 1;
 			continue;
 		} else if G_UNLIKELY (*p != 'F' && *p != 'V' && *p != 'f' && *p != 'v') {
 			gel_errorout (_("Badly formed record"));
@@ -1907,7 +1915,7 @@ load_compiled_fp (const char *file, FILE *fp)
 			extra_dict = FALSE;
 
 		/*size*/
-		p = strtok(NULL,";");
+		p = strtok_r (NULL,";", &ptrptr);
 		if G_UNLIKELY (!p) {
 			gel_errorout (_("Badly formed record"));
 			continue;
@@ -1920,7 +1928,7 @@ load_compiled_fp (const char *file, FILE *fp)
 		}
 
 		/*id*/
-		p = strtok(NULL,";");
+		p = strtok_r (NULL,";", &ptrptr);
 		if G_UNLIKELY (!p) {
 			gel_errorout (_("Badly formed record"));
 			continue;
@@ -1928,8 +1936,19 @@ load_compiled_fp (const char *file, FILE *fp)
 		tok = d_intern(p);
 
 		if (type == GEL_USER_FUNC) {
+			/*symbolic_id*/
+			p = strtok_r (NULL,";", &ptrptr);
+			if G_UNLIKELY (!p) {
+				gel_errorout (_("Badly formed record"));
+				continue;
+			}
+			if (strcmp (p, "*") == 0)
+				symbolic_tok = NULL;
+			else
+				symbolic_tok = d_intern (p);
+
 			/*nargs*/
-			p = strtok(NULL,";");
+			p = strtok_r (NULL,";", &ptrptr);
 			if G_UNLIKELY (!p) {
 				gel_errorout (_("Badly formed record"));
 				continue;
@@ -1942,7 +1961,7 @@ load_compiled_fp (const char *file, FILE *fp)
 			}
 
 			/*vararg*/
-			p = strtok(NULL,";");
+			p = strtok_r (NULL,";", &ptrptr);
 			if (p == NULL) {
 				gel_errorout (_("Badly formed record"));
 				continue;
@@ -1957,7 +1976,7 @@ load_compiled_fp (const char *file, FILE *fp)
 			/*argument names*/
 			li = NULL;
 			for(i=0;i<nargs;i++) {
-				p = strtok(NULL,";");
+				p = strtok_r (NULL,";", &ptrptr);
 				if G_UNLIKELY (p == NULL) {
 					gel_errorout (_("Badly formed record"));
 					g_slist_free(li);
@@ -1967,7 +1986,7 @@ load_compiled_fp (const char *file, FILE *fp)
 			}
 		} else {
 			/*parameter*/
-			p = strtok(NULL,";");
+			p = strtok_r (NULL,";", &ptrptr);
 			if G_UNLIKELY (!p) {
 				gel_errorout (_("Badly formed record"));
 				continue;
@@ -1980,7 +1999,6 @@ load_compiled_fp (const char *file, FILE *fp)
 				tok->parameter = 1;
 		}
 
-
 		/*the value*/
 		b2 = g_new(char,size+2);
 		if G_UNLIKELY (!fgets(b2,size+2,fp)) {
@@ -1998,6 +2016,7 @@ load_compiled_fp (const char *file, FILE *fp)
 			if (type == GEL_USER_FUNC) {
 				func = d_makeufunc (tok, NULL, li, nargs, NULL);
 				func->vararg = vararg ? 1 : 0;
+				func->symbolic_id = symbolic_tok;
 			} else /*GEL_VARIABLE_FUNC*/ {
 				func = d_makevfunc (tok, NULL);
 			}
@@ -2015,6 +2034,7 @@ load_compiled_fp (const char *file, FILE *fp)
 			if(type == GEL_USER_FUNC) {
 				func = d_makeufunc (tok, NULL, li, nargs, NULL);
 				func->vararg = vararg ? 1 : 0;
+				func->symbolic_id = symbolic_tok;
 			} else /*GEL_VARIABLE_FUNC*/ {
 				func = d_makevfunc(tok,NULL);
 			}
@@ -2145,6 +2165,60 @@ print_function_help (GelHelp *help)
 		f = make_function_with_aliases (help->func, help->aliases);
 		len = strlen (f);
 		do_cyan ();
+
+#if 0
+/* This can be used to autogenerate some docbook */
+		gel_output_printf_full (main_out, FALSE,
+					"        <varlistentry id=\"gel-function-%s\">\n"
+					"         <term>%s</term>\n"
+					"         <listitem>\n"
+					"          <synopsis>",
+					help->func, help->func);
+		{
+			GelEFunc *ff;
+			ff = d_lookup_global (d_intern (help->func));
+			if (ff == NULL || (ff->type == GEL_BUILTIN_FUNC &&
+					   ff->named_args == NULL &&
+					   ! ff->vararg) ||
+			    		  (ff->named_args == NULL &&
+					   d_intern (help->func)->parameter)) {
+				gel_output_printf_full (main_out, FALSE, "%s</synopsis>\n", help->func);
+			} else {
+				GSList *li;
+				gel_output_printf_full (main_out, FALSE, "%s (", help->func);
+
+				for (li = ff->named_args; li != NULL; li = li->next) {
+					GelToken *id = li->data;
+					if (li != ff->named_args)
+						gel_output_full_string (main_out, ",");
+					gel_output_full_string (main_out, id->token);
+				}
+
+				if (ff->vararg)
+					gel_output_full_string (main_out, "...");
+				gel_output_full_string (main_out, ")</synopsis>\n");
+			}
+			gel_output_printf_full (main_out, FALSE, "          <para>");
+			if (help->aliases != NULL) {
+				GSList *li;
+				GString *gs = g_string_new ("Aliases:");
+				for (li = help->aliases; li != NULL; li = li->next) {
+					g_string_append (gs, " <function>");
+					g_string_append (gs, li->data);
+					g_string_append (gs, "</function>");
+				}
+				gel_output_printf_full (main_out, FALSE,
+							"%s</para>\n          <para>", gs->str);
+				g_string_free (gs, TRUE);
+			}
+			gel_output_printf_full (main_out, FALSE,
+						"%s</para>\n         </listitem>\n        </varlistentry>\n\n",
+						help->description);
+		}
+#endif
+
+
+
 		/*if (len <= 20)*/
 			gel_output_printf_full (main_out, FALSE,
 						"%-20s", f);
@@ -2157,7 +2231,7 @@ print_function_help (GelHelp *help)
 		do_green ();
 		if (help->description != NULL)
 			print_description (MAX (20, len),
-					   help->description);
+					   _(help->description));
 		else
 			gel_output_full_string (main_out, "\n");
 		/* if we didn't fit aliases on one line */
@@ -2318,8 +2392,8 @@ full_help (void)
 	gel_output_pop_nonotify (main_out);
 }
 
-static void
-help_on (const char *text)
+void
+gel_help_on (const char *text)
 {
 	GelHelp *help;
 	GelHelp not_documented = { NULL /* func */,
@@ -2337,6 +2411,7 @@ help_on (const char *text)
 	for (i = 0; genius_toplevels[i] != NULL; i++)
 		if (strcmp (text, genius_toplevels[i]) == 0) {
 			print_command_help (text);
+			gel_call_help (text);
 			do_black ();
 			gel_output_pop_nonotify (main_out);
 			return;
@@ -2344,7 +2419,14 @@ help_on (const char *text)
 
 	help = get_help (text, FALSE /*insert*/);
 	if (help == NULL) {
-		gel_errorout (_("'%s' is not documented"), text);
+		char *similar_ids = gel_similar_possible_ids (text);
+		if (similar_ids == NULL) {
+			gel_errorout (_("'%s' is not documented"), text);
+		} else {
+			gel_errorout (_("'%s' is not documented.  Perhaps "
+					"you meant %s."), text, similar_ids);
+			g_free (similar_ids);
+		}
 		not_documented.func = (char *)text;
 		help = &not_documented;
 	}
@@ -2353,7 +2435,7 @@ help_on (const char *text)
 		gel_output_printf_full (main_out, FALSE,
 					_("%s is an alias for %s\n"),
 					text, help->aliasfor);
-		help_on (help->aliasfor);
+		gel_help_on (help->aliasfor);
 		do_black ();
 		gel_output_pop_nonotify (main_out);
 		return;
@@ -2362,7 +2444,14 @@ help_on (const char *text)
 	do_cyan ();
 
 	f = d_lookup_global (d_intern (text));
-	if (f == NULL) {
+
+	if (d_intern (text)->parameter) {
+		gel_output_printf_full (main_out, FALSE, "%s%s\n",
+					_("Parameter: "), text);
+	} else if (f == NULL
+		   || (f->type == GEL_BUILTIN_FUNC &&
+		       f->named_args == NULL &&
+		       ! f->vararg)) {
 		gel_output_printf_full (main_out, FALSE, "%s\n", text);
 	} else {
 		GSList *li;
@@ -2396,13 +2485,82 @@ help_on (const char *text)
 	if (help->description != NULL) {
 		gel_output_printf_full (main_out, FALSE,
 					_("Description: %s\n"),
-					help->description);
+					_(help->description));
 	}
 
 	do_black ();
 	gel_output_pop_nonotify (main_out);
+
+	if (help != &not_documented)
+		gel_call_help (text);
 }
 
+static void
+dump_a_string (FILE *outfile, const char *s)
+{
+	fprintf (outfile, "char *fake = N_(\"%s\");\n", s);
+}
+
+void
+gel_dump_strings_from_user_funcs (FILE *outfile)
+{
+	/* FIXME: implement */
+}
+
+static void
+dump_cat (FILE *outfile, const char *cat)
+{
+	GSList *functions;
+	GSList *fli;
+
+	functions = get_helps (cat);
+
+	if (functions != NULL) {
+		for (fli = functions; fli != NULL; fli = fli->next) {
+			GelHelp *help = fli->data;
+			GelToken *id = d_intern (help->func);
+			GelEFunc *f = d_lookup_global (id);
+
+			if (help->description != NULL &&
+			    (f == NULL ||
+			     f->type != GEL_BUILTIN_FUNC) &&
+			    ! id->built_in_parameter) {
+				dump_a_string (outfile, help->description);
+			}
+
+			fli->data = NULL;
+		}
+
+		g_slist_free (functions);
+	}
+}
+
+void
+gel_dump_strings_from_help (FILE *outfile)
+{
+	GSList *categories = get_categories ();
+	GSList *cli;
+
+	for (cli = categories; cli != NULL; cli = cli->next) {
+		char *cat = cli->data;
+		HelpCategory *cats;
+
+		cats = get_category (cat, FALSE /* insert */);
+		if (cats != NULL &&
+		    cats->name != NULL &&
+		    ! cats->internal) {
+			dump_a_string (outfile, cats->name);
+		}
+
+		dump_cat (outfile, cat);
+
+		cli->data = NULL;
+		g_free (cat);
+	}
+	g_slist_free (categories);
+
+	dump_cat (outfile, NULL);
+}
 
 void
 set_new_calcstate(calcstate_t state)
@@ -2527,7 +2685,7 @@ static GSList *
 get_wordlist (const char *lst)
 {
 	GSList *list = NULL;
-#if HAVE_WORDEXP
+#ifdef HAVE_WORDEXP
 	wordexp_t we;
 	int i;
 	if G_UNLIKELY (wordexp (lst, &we, WRDE_NOCMD) != 0) {
@@ -2572,15 +2730,15 @@ do_exec_commands (const char *dirprefix)
 		ret = FALSE;
 		break;
 	case GEL_LOADFILE:
-		while (evalstack)
-			gel_freetree (stack_pop (&evalstack));
+		while (gel_parsestack)
+			gel_freetree (stack_pop (&gel_parsestack));
 		gel_load_file (dirprefix, arg, TRUE);
 		ret = TRUE;
 		break;
 	case GEL_LOADFILE_GLOB:
 		list = get_wordlist (arg);
-		while (evalstack)
-			gel_freetree (stack_pop (&evalstack));
+		while (gel_parsestack)
+			gel_freetree (stack_pop (&gel_parsestack));
 		for (li = list; li != NULL; li = li->next) {
 			gel_load_guess_file (dirprefix, li->data, TRUE);
 			if (interrupted)
@@ -2592,8 +2750,8 @@ do_exec_commands (const char *dirprefix)
 		break;
 	case GEL_CHANGEDIR:
 		list = get_wordlist (arg);
-		while (evalstack)
-			gel_freetree (stack_pop (&evalstack));
+		while (gel_parsestack)
+			gel_freetree (stack_pop (&gel_parsestack));
 		for (li = list; li != NULL; li = li->next) {
 			our_chdir (dirprefix, li->data);
 		}
@@ -2712,7 +2870,7 @@ do_exec_commands (const char *dirprefix)
 		break;
 	case GEL_HELP_ARG:
 		g_strstrip (arg);
-		help_on (arg);
+		gel_help_on (arg);
 		ret = TRUE;
 		break;
 	default:
@@ -2723,12 +2881,10 @@ do_exec_commands (const char *dirprefix)
 	return ret;
 }
 
-GelETree *
-gel_parseexp(const char *str, FILE *infile, gboolean exec_commands, gboolean testparse,
-	 gboolean *finished, const char *dirprefix)
+/* run this before gel_parseexp on a standalone run */
+void
+gel_execinit (void)
 {
-	int stacklen;
-
 	interrupted = FALSE;
 
 	/*init the context stack and clear out any stale dictionaries
@@ -2736,6 +2892,13 @@ gel_parseexp(const char *str, FILE *infile, gboolean exec_commands, gboolean tes
 	  will also register the builtin routines with the global
 	  dictionary*/
 	d_singlecontext();
+}
+
+GelETree *
+gel_parseexp (const char *str, FILE *infile, gboolean exec_commands,
+	      gboolean testparse, gboolean *finished, const char *dirprefix)
+{
+	int stacklen;
 
 	error_num = NO_ERROR;
 
@@ -2791,21 +2954,21 @@ gel_parseexp(const char *str, FILE *infile, gboolean exec_commands, gboolean tes
 
 	/*if we are testing and got an unfinished expression just report that*/
 	if(testparse && got_end_too_soon) {
-		while(evalstack)
-			gel_freetree(stack_pop(&evalstack));
+		while(gel_parsestack)
+			gel_freetree(stack_pop(&gel_parsestack));
 		if(finished) *finished = FALSE;
 		return NULL;
 	}
 
 	/*catch parsing errors*/
 	if(error_num!=NO_ERROR) {
-		while(evalstack)
-			gel_freetree(stack_pop(&evalstack));
+		while(gel_parsestack)
+			gel_freetree(stack_pop(&gel_parsestack));
 		if(finished) *finished = TRUE;
 		return NULL;
 	}
 	
-	stacklen = g_slist_length(evalstack);
+	stacklen = g_slist_length(gel_parsestack);
 	
 	if(stacklen==0) {
 		if(finished) *finished = FALSE;
@@ -2814,22 +2977,22 @@ gel_parseexp(const char *str, FILE *infile, gboolean exec_commands, gboolean tes
 
 	/*stack is supposed to have only ONE entry*/
 	if(stacklen!=1) {
-		while(evalstack)
-			gel_freetree(stack_pop(&evalstack));
+		while(gel_parsestack)
+			gel_freetree(stack_pop(&gel_parsestack));
 		if G_UNLIKELY (!testparse)
 			gel_errorout (_("ERROR: Probably corrupt stack!"));
 		if(finished) *finished = FALSE;
 		return NULL;
 	}
-	replace_equals (evalstack->data, FALSE /* in_expression */);
-	replace_exp (evalstack->data);
-	fixup_num_neg (evalstack->data);
-	evalstack->data = gather_comparisons (evalstack->data);
-	try_to_do_precalc (evalstack->data);
+	replace_equals (gel_parsestack->data, FALSE /* in_expression */);
+	replace_exp (gel_parsestack->data);
+	fixup_num_neg (gel_parsestack->data);
+	gel_parsestack->data = gather_comparisons (gel_parsestack->data);
+	try_to_do_precalc (gel_parsestack->data);
 	
 	if (finished != NULL)
 		*finished = TRUE;
-	return stack_pop (&evalstack);
+	return stack_pop (&gel_parsestack);
 }
 
 GelETree *
@@ -2851,7 +3014,7 @@ gel_runexp (GelETree *exp)
 	gel_push_file_info(NULL,0);
 
 	ctx = eval_get_context();
-	ret = eval_etree(ctx,copynode(exp));
+	ret = eval_etree (ctx, exp);
 	eval_free_context(ctx);
 
 	gel_pop_file_info();
@@ -2868,6 +3031,7 @@ gel_runexp (GelETree *exp)
 	return ret;
 }
 
+/* 'parsed' is eaten */
 void
 gel_evalexp_parsed (GelETree *parsed,
 		    GelOutput *gelo,
@@ -2878,8 +3042,8 @@ gel_evalexp_parsed (GelETree *parsed,
 	
 	if (parsed == NULL)
 		return;
+	/* gel_runexp eats the 'parsed' */
 	ret = gel_runexp (parsed);
-	gel_freetree (parsed);
 	if (ret == NULL)
 		return;
 
@@ -2926,6 +3090,7 @@ gel_evalexp (const char *str,
 	     const char *dirprefix)
 {
 	GelETree *parsed;
+	gel_execinit ();
 	parsed = gel_parseexp (str, infile, TRUE, FALSE, NULL, dirprefix);
 	gel_evalexp_parsed (parsed, gelo, prefix, pretty);
 }
