@@ -45,36 +45,47 @@ static GHashTable *dictionary;
 
 extern GHashTable *uncompiled;
 
+extern const char *genius_toplevels[];
+extern const char *genius_operators[];
+
 GelEFunc *free_funcs = NULL;
 
+#define GET_NEW_FUNC(n) \
+	if (free_funcs == NULL) {			\
+		n = g_new (GelEFunc, 1);		\
+	} else {					\
+		n = free_funcs;				\
+		free_funcs = free_funcs->data.next;	\
+	}
+
 /*return current context number (0 is global, -1 is uninitialized)*/
 int
-d_curcontext(void)
+d_curcontext (void)
 {
 	return context.top;
 }
 
 /*make builtin function and return it*/
 GelEFunc *
-d_makebifunc(GelToken *id, dictfunc f, int nargs)
+d_makebifunc (GelToken *id, GelBIFunction f, int nargs)
 {
 	GelEFunc *n;
 
-	if(!free_funcs)
-		n = g_new(GelEFunc,1);
-	else {
-		n = free_funcs;
-		free_funcs = free_funcs->data.next;
-	}
+	GET_NEW_FUNC (n);
+	memset (n, 0, sizeof (GelEFunc));
 	n->id = id;
 	n->data.func = f;
 	n->nargs = nargs;
+	n->context = context.top;
+	n->type = GEL_BUILTIN_FUNC;
+	/*
 	n->vararg = 0;
 	n->on_subst_list = 0;
 	n->named_args = NULL;
 	n->extra_dict = NULL;
-	n->context = context.top;
-	n->type = GEL_BUILTIN_FUNC;
+	n->no_mod_all_args = FALSE;
+	n->propagate_mod = FALSE;
+	*/
 
 	return n;
 }
@@ -86,28 +97,29 @@ d_makeufunc (GelToken *id, GelETree *value, GSList *argnames, int nargs,
 {
 	GelEFunc *n;
 
-	if(!free_funcs)
-		n = g_new(GelEFunc,1);
-	else {
-		n = free_funcs;
-		free_funcs = free_funcs->data.next;
-	}
-	n->id=id;
-	n->data.user=value;
-	n->nargs=nargs;
+	GET_NEW_FUNC (n);
+	memset (n, 0, sizeof (GelEFunc));
+
+	n->id = id;
+	n->data.user = value;
+	n->nargs = nargs;
+	n->named_args = argnames;
+	n->context = context.top;
+	n->type = GEL_USER_FUNC;
+
+	/* look at the memset
 	n->vararg = 0;
 	n->on_subst_list = 0;
-	n->named_args=argnames;
-	n->context=context.top;
-	n->type=GEL_USER_FUNC;
+	n->no_mod_all_args = FALSE;
+	n->propagate_mod = FALSE;
+	n->extra_dict = NULL;
+	*/
 
 	if (extra_dict != NULL) {
 		GSList *li;
 		n->extra_dict = g_slist_copy ((GSList *)extra_dict);
 		for (li = n->extra_dict; li != NULL; li = li->next)
 			li->data = d_copyfunc (li->data);
-	} else {
-		n->extra_dict = NULL;
 	}
 
 	return n;
@@ -119,21 +131,23 @@ d_makevfunc(GelToken *id, GelETree *value)
 {
 	GelEFunc *n;
 
-	if(!free_funcs)
-		n = g_new(GelEFunc,1);
-	else {
-		n = free_funcs;
-		free_funcs = free_funcs->data.next;
-	}
-	n->id=id;
-	n->data.user=value;
-	n->nargs=0;
+	GET_NEW_FUNC (n);
+	memset (n, 0, sizeof (GelEFunc));
+
+	n->id = id;
+	n->data.user = value;
+	n->context = context.top;
+	n->type = GEL_VARIABLE_FUNC;
+
+	/* look at the memset
+	n->nargs = 0;
 	n->vararg = 0;
 	n->on_subst_list = 0;
-	n->named_args=NULL;
+	n->named_args = NULL;
 	n->extra_dict = NULL;
-	n->context=context.top;
-	n->type=GEL_VARIABLE_FUNC;
+	n->no_mod_all_args = FALSE;
+	n->propagate_mod = FALSE;
+	*/
 
 	return n;
 }
@@ -144,21 +158,23 @@ d_makereffunc(GelToken *id, GelEFunc *ref)
 {
 	GelEFunc *n;
 
-	if(!free_funcs)
-		n = g_new(GelEFunc,1);
-	else {
-		n = free_funcs;
-		free_funcs = free_funcs->data.next;
-	}
+	GET_NEW_FUNC (n);
+	memset (n, 0, sizeof (GelEFunc));
+
 	n->id = id;
 	n->data.ref = ref;
+	n->context = context.top;
+	n->type = GEL_REFERENCE_FUNC;
+
+	/* look at the memset
 	n->nargs = 0;
 	n->vararg = 0;
 	n->on_subst_list = 0;
 	n->named_args = NULL;
 	n->extra_dict = NULL;
-	n->context = context.top;
-	n->type = GEL_REFERENCE_FUNC;
+	n->no_mod_all_args = FALSE;
+	n->propagate_mod = FALSE;
+	*/
 
 	return n;
 }
@@ -170,13 +186,9 @@ d_copyfunc(GelEFunc *o)
 	GelEFunc *n;
 	GSList *li;
 
-	if(!free_funcs)
-		n = g_new(GelEFunc,1);
-	else {
-		n = free_funcs;
-		free_funcs = free_funcs->data.next;
-	}
-	memcpy(n,o,sizeof(GelEFunc));
+	GET_NEW_FUNC (n);
+	memcpy (n, o, sizeof (GelEFunc));
+
 	if(n->type == GEL_USER_FUNC ||
 	   n->type == GEL_VARIABLE_FUNC) {
 		D_ENSURE_USER_BODY (o);
@@ -198,18 +210,17 @@ d_copyfunc(GelEFunc *o)
 
 /*make a real function from a fake*/
 GelEFunc *
-d_makerealfunc(GelEFunc *o,GelToken *id, int use)
+d_makerealfunc(GelEFunc *o,GelToken *id, gboolean use)
 {
 	GelEFunc *n;
 
-	if(!free_funcs)
-		n = g_new(GelEFunc,1);
-	else {
-		n = free_funcs;
-		free_funcs = free_funcs->data.next;
-	}
-	memcpy(n,o,sizeof(GelEFunc));
+	GET_NEW_FUNC (n);
+	memcpy (n, o, sizeof (GelEFunc));
 	n->id = id;
+	if (o->symbolic_id == NULL)
+		n->symbolic_id = o->id;
+	else
+		n->symbolic_id = o->symbolic_id;
 	n->context = context.top;
 
 	if(n->type == GEL_USER_FUNC ||
@@ -247,7 +258,7 @@ d_makerealfunc(GelEFunc *o,GelToken *id, int use)
 /*make real func and replace n with it, without changing n's context or id*/
 /*if use is set, we USE the original function, NULLing approriately*/
 void
-d_setrealfunc(GelEFunc *n,GelEFunc *fake, int use)
+d_setrealfunc(GelEFunc *n,GelEFunc *fake, gboolean use)
 {
 	if(n->type == GEL_USER_FUNC ||
 	   n->type == GEL_VARIABLE_FUNC)
@@ -255,6 +266,9 @@ d_setrealfunc(GelEFunc *n,GelEFunc *fake, int use)
 	
 	n->type = fake->type;
 	n->data = fake->data;
+	n->symbolic_id = fake->symbolic_id;
+	if (fake->symbolic_id == NULL)
+		n->symbolic_id = fake->id;
 	if(n->type == GEL_USER_FUNC ||
 	   n->type == GEL_VARIABLE_FUNC) {
 		D_ENSURE_USER_BODY (fake);
@@ -329,7 +343,7 @@ d_addfunc (GelEFunc *func)
 {
 	GelEFunc *n;
 	
-	g_return_val_if_fail(func->context == context.top,func);
+	g_return_val_if_fail (func->context == context.top, func);
 	
 	/*we already found it (in current context)*/
 	n = d_lookup_local(func->id);
@@ -378,8 +392,8 @@ d_addfunc_global (GelEFunc *func)
 
 /*set value of an existing function (in local context), used for arguments
   WARNING, does not free the memory allocated by previous value!*/
-int
-d_setvalue(GelToken *id,GelETree *value)
+gboolean
+d_setvalue (GelToken *id, GelETree *value)
 {
 	GelEFunc *f;
 	f=d_lookup_local(id);
@@ -460,16 +474,22 @@ d_lookup_only_global (GelToken *id)
 GelToken *
 d_intern (const char *id)
 {
-	GelToken * tok = g_hash_table_lookup(dictionary,id);
-	if(!tok) {
-		tok = g_new0(GelToken,1);
-		tok->token = g_strdup(id);
-		g_hash_table_insert(dictionary,tok->token,tok);
+	GelToken * tok;
+
+	if (id == NULL)
+		return NULL;
+
+        tok = g_hash_table_lookup (dictionary, id);
+	if (tok == NULL) {
+		tok = g_new0 (GelToken, 1);
+		tok->token = g_strdup (id);
+		g_hash_table_insert (dictionary, tok->token, tok);
 	}
+
 	return tok;
 }
 
-int
+gboolean
 d_delete(GelToken *id)
 {
 	/*FIXME: Delete function!*/
@@ -569,16 +589,16 @@ d_freefunc (GelEFunc *n)
   new has to have the same id, also new->id should
   not hold a reference to new*/
 void
-d_replacefunc(GelEFunc *old,GelEFunc *new)
+d_replacefunc(GelEFunc *old,GelEFunc *_new)
 {
 	GSList *li;
 
 	/* assert some things we don't deal with.  Should we? */
-	g_assert ( ! new->on_subst_list);
+	g_assert ( ! _new->on_subst_list);
 	g_assert ( ! old->on_subst_list);
 
-	g_return_if_fail(old && new);
-	g_return_if_fail(old->id == new->id);
+	g_return_if_fail(old && _new);
+	g_return_if_fail(old->id == _new->id);
 
 	if(old->type == GEL_USER_FUNC ||
 	   old->type == GEL_VARIABLE_FUNC)
@@ -592,14 +612,14 @@ d_replacefunc(GelEFunc *old,GelEFunc *new)
 	}
 	g_slist_free (old->extra_dict);
 
-	memcpy(old,new,sizeof(GelEFunc));
+	memcpy(old,_new,sizeof(GelEFunc));
 
 #ifndef MEM_DEBUG_FRIENDLY
 	/*prepend to free list*/
-	new->data.next = free_funcs;
-	free_funcs = new;
+	_new->data.next = free_funcs;
+	free_funcs = _new;
 #else
-	g_free (new);
+	g_free (_new);
 #endif
 }
 
@@ -638,7 +658,7 @@ d_set_value(GelEFunc *n,GelETree *value)
 }
 
 /*push a new dictionary onto the context stack*/
-int
+gboolean
 d_addcontext(void)
 {
 	context.stack = g_slist_prepend (context.stack, NULL);
@@ -700,14 +720,184 @@ d_popcontext(void)
 
 /*gimme the current dictinary*/
 GSList *
-d_getcontext(void)
+d_getcontext (void)
 {
-	if(context.top==-1)
+	if (context.top == -1)
 		return NULL;
 	else
 		return context.stack->data;
 }
 
+/*gimme the current global dictinary*/
+GSList *
+d_getcontext_global (void)
+{
+	if (context.top == -1) {
+		return NULL;
+	} else {
+		GSList *last;
+		last = g_slist_last (context.stack);
+		g_assert (last != NULL);
+
+		return last->data;
+	}
+}
+
+static int
+lowercase_ascii_sum (const char *id)
+{
+	int sum = 0;
+	int i;
+	for (i = 0; id[i] != '\0'; i++) {
+		sum += g_ascii_tolower (id[i]) - 'a';
+	}
+	return sum;
+}
+
+static int
+lowercase_kronecker_difference (const char *id1, const char *id2)
+{
+	int sum = 0;
+	int i;
+	for (i = 0; id1[i] != '\0' && id2[i] != '\0'; i++) {
+		if (g_ascii_tolower (id1[i]) != g_ascii_tolower (id2[i]))
+			sum += 1;
+	}
+	/* plus the ends */
+	sum += strlen (&id1[i]);
+	sum += strlen (&id2[i]);
+	return sum;
+}
+
+static char *
+construct_deleted (const char *id, int d, int len)
+{
+	int i, y;
+	char *out = g_new0 (char, len);
+
+	y = 0;
+	for (i = 0; i < len; i++) {
+		if (i != d)
+			out[y++] = id[i];
+	}
+
+	return out;
+}
+
+static gboolean
+try_deletions (const char *id1, int len1, const char *id2)
+{
+	int i;
+	for (i = 0; i < len1; i++) {
+		char *d = construct_deleted (id1, i, len1);
+		int allowed = len1 * 0.2;
+		if (lowercase_kronecker_difference (id2, d) <= allowed) {
+			g_free (d);
+			return TRUE;
+		}
+		g_free (d);
+	}
+
+	return FALSE;
+}
+
+static gboolean
+are_ids_similar (const char *id1, const char *id2)
+{
+	int len1 = strlen (id1), len2 = strlen (id2);
+	int dif1, dif2, dif3;
+	int allowed;
+
+	if (ABS (len1 - len2) > 1)
+		return FALSE;
+
+	if (g_ascii_strcasecmp (id1, id2) == 0)
+		return TRUE;
+
+	if (len1 > 6 && len1 == len2) {
+		int sum1, sum2;
+
+		sum1 = lowercase_ascii_sum (id1);
+		sum2 = lowercase_ascii_sum (id2);
+
+		/* just a reordering (possibly)
+		   (won't work right on small words) */
+		if (sum1 == sum2) {
+			return TRUE;
+		}
+	}
+
+	/* simple cases for length 2 and 3 */
+	if (len1 == 2 &&
+	    len2 == len1 &&
+	    id1[0] == id2[1] &&
+	    id1[1] == id2[0]) {
+		return TRUE;
+	}
+
+	if (len1 == 3 &&
+	    len2 == len1 &&
+	    ( (id1[0] == id2[1] && id1[1] == id2[0] && id1[2] == id2[2]) ||
+	      (id1[0] == id2[0] && id1[1] == id2[2] && id1[2] == id2[1]) )) {
+		return TRUE;
+	}
+
+	if (len1 <= 2 && len2 <= 2) {
+		return FALSE;
+	}
+
+	if (try_deletions (id1, len1, id2))
+		return TRUE;
+	if (try_deletions (id2, len2, id1))
+		return TRUE;
+
+	dif1 = lowercase_kronecker_difference (id1, id2);
+	dif2 = lowercase_kronecker_difference (id1, &id2[1]);
+	dif3 = lowercase_kronecker_difference (&id1[1], id2);
+
+	allowed = 0.2 * len1;
+
+	if (dif1 > allowed+1 &&
+	    dif2 > allowed &&
+	    dif3 > allowed)
+		return FALSE;
+
+	return TRUE;
+}
+
+GSList *
+d_find_similar_globals (const char *id)
+{
+	GSList *ret = NULL;
+	GSList *li;
+	int i;
+
+	for (li = d_getcontext_global (); li != NULL; li = li->next) {
+		GelEFunc *n = li->data;
+		if (n->id != NULL &&
+		    n->id->token != NULL &&
+		    are_ids_similar (n->id->token, id)) {
+			ret = g_slist_prepend (ret, g_strdup (n->id->token));
+		}
+	}
+
+	for (i = 0; genius_toplevels[i] != NULL; i++) {
+		if (are_ids_similar (genius_toplevels[i], id)) {
+			ret = g_slist_prepend (ret,
+					       g_strdup (genius_toplevels[i]));
+		}
+	}
+
+	for (i = 0; genius_operators[i] != NULL; i++) {
+		if (are_ids_similar (genius_operators[i], id)) {
+			ret = g_slist_prepend (ret,
+					       g_strdup (genius_operators[i]));
+		}
+	}
+
+	return ret;
+}
+
 void
 d_protect_all(void)
 {
@@ -720,7 +910,7 @@ d_protect_all(void)
 		if(!func->id || strcmp(func->id->token,"Ans")==0)
 			continue;
 		if ( ! func->id->parameter)
-			func->id->protected = 1;
+			func->id->protected_ = 1;
 	}
 }
 
