@@ -1,5 +1,5 @@
-/* GnomENIUS Calculator
- * Copyright (C) 1999 the Free Software Foundation.
+/* GENIUS Calculator
+ * Copyright (C) 1997-2002 George Lebl
  *
  * Author: George Lebl
  *
@@ -19,21 +19,31 @@
  * USA.
  */
 
-#ifndef _PLUGIN_H_
-#define _PLUGIN_H_
+#ifndef PLUGIN_H__
+#define PLUGIN_H__
 
-typedef struct _plugin_t {
+typedef struct _GelPlugin {
+	char *base;
 	char *file;
 	char *name;
 	char *author;
 	char *copyright;
 	char *description;
-} plugin_t;
+	gboolean gui;
+	gboolean hide;
+
+	/* flags while running */
+	gboolean running;
+	gboolean restore;
+	char *unique_id;
+} GelPlugin;
 
 /*read or reread the plugin list from the share/genius/plugins directory*/
-void read_plugin_list(void);
-extern GList *plugin_list;
+void gel_read_plugin_list(void);
+extern GSList *gel_plugin_list;
 
-void open_plugin(plugin_t *plug);
+void gel_open_plugin (GelPlugin *plug);
+void gel_save_plugins (void);
+void gel_restore_plugins (void);
 
-#endif
+#endif /* PLUGIN_H__ */
